@@ -1,13 +1,14 @@
 from __future__ import annotations
 
-import itertools
-from abc import ABC
+from abc import ABC, abstractmethod
 from collections import OrderedDict, deque
 from collections.abc import (
     AsyncGenerator,
     AsyncIterator,
     Awaitable,
     Callable,
+    Collection,
+    Container,
     Generator,
     Iterable,
     Iterator,
@@ -16,8 +17,9 @@ from collections.abc import (
 from contextlib import asynccontextmanager, contextmanager, suppress
 from dataclasses import dataclass, field
 from enum import StrEnum
-from functools import partial, partialmethod, singledispatchmethod, update_wrapper
+from functools import partialmethod, singledispatchmethod, update_wrapper
 from inspect import (
+    BoundArguments,
     Signature,
     isasyncgenfunction,
     isclass,
@@ -42,21 +44,17 @@ from typing import (
 
 from type_analyzer import MatchingTypesConfig, iter_matching_types, matching_types
 
-from injection._core.common.asynchronous import (
-    Caller,
-    HiddenCaller,
-    SimpleAwaitable,
-)
+from injection._core.common.asynchronous import Caller, SimpleAwaitable
 from injection._core.common.event import Event, EventChannel, EventListener
 from injection._core.common.invertible import Invertible, SimpleInvertible
-from injection._core.common.key import new_short_key
 from injection._core.common.lazy import Lazy
 from injection._core.common.threading import get_lock
 from injection._core.common.type import (
     InputType,
     TypeInfo,
-    get_return_types,
-    get_yield_hint,
+    get_yield_types,
+    iter_flat_types,
+    iter_return_types,
 )
 from injection._core.injectables import (
     AsyncCMScopedInjectable,
@@ -91,10 +89,6 @@ from injection.exceptions import (
     SkipInjectable,
 )
 
-"""
-Events
-"""
-
 
 @dataclass(frozen=True, slots=True)
 class ModuleEvent(Event, ABC):
@@ -151,11 +145,6 @@ class ModulePriorityUpdated(ModuleEvent):
         )
 
 
-"""
-Module
-"""
-
-
 class Priority(StrEnum):
     LOW = "low"
     HIGH = "high"
@@ -167,7 +156,7 @@ class Priority(StrEnum):
 
 type PriorityStr = Literal["low", "high"]
 
-type ContextManagerLikeRecipe[**P, T] = (
+type ContextManagerRecipe[**P, T] = (
     Callable[P, ContextManager[T]] | Callable[P, AsyncContextManager[T]]
 )
 type GeneratorRecipe[**P, T] = (
@@ -175,9 +164,16 @@ type GeneratorRecipe[**P, T] = (
 )
 
 
+@dataclass(repr=False, eq=False, frozen=True, slots=True)
+class _ScopedContext[**P, T]:
+    cls: type[ScopedInjectable[Any, T]]
+    hints: Collection[TypeInfo[T]]
+    wrapper: Recipe[P, T] | ContextManagerRecipe[P, T]
+
+
 @dataclass(eq=False, frozen=True, slots=True)
 class Module(EventListener, InjectionProvider):  # type: ignore[misc]
-    name: str = field(default_factory=lambda: f"anonymous@{new_short_key()}")
+    name: str | None = field(default=None)
     __channel: EventChannel = field(
         default_factory=EventChannel,
         init=False,
@@ -257,6 +253,7 @@ class Module(EventListener, InjectionProvider):  # type: ignore[misc]
         scope_name: str,
         /,
         *,
+        ignore_type_hint: bool = False,
         inject: bool = True,
         on: TypeInfo[T] = (),
         mode: Mode | ModeStr = Mode.get_default(),
@@ -264,29 +261,33 @@ class Module(EventListener, InjectionProvider):  # type: ignore[misc]
         def decorator(
             wrapped: Recipe[P, T] | GeneratorRecipe[P, T],
         ) -> Recipe[P, T] | GeneratorRecipe[P, T]:
-            injectable_class: type[ScopedInjectable[Any, T]]
-            wrapper: Recipe[P, T] | ContextManagerLikeRecipe[P, T]
-
             if isasyncgenfunction(wrapped):
-                hint = get_yield_hint(wrapped)
-                injectable_class = AsyncCMScopedInjectable
-                wrapper = asynccontextmanager(wrapped)
+                ctx = _ScopedContext(
+                    cls=AsyncCMScopedInjectable,
+                    hints=() if ignore_type_hint else get_yield_types(wrapped),
+                    wrapper=asynccontextmanager(wrapped),
+                )
 
             elif isgeneratorfunction(wrapped):
-                hint = get_yield_hint(wrapped)
-                injectable_class = CMScopedInjectable
-                wrapper = contextmanager(wrapped)
+                ctx = _ScopedContext(
+                    cls=CMScopedInjectable,
+                    hints=() if ignore_type_hint else get_yield_types(wrapped),
+                    wrapper=contextmanager(wrapped),
+                )
 
             else:
-                injectable_class = SimpleScopedInjectable
-                hint = wrapper = wrapped  # type: ignore[assignment]
+                ctx = _ScopedContext(
+                    cls=SimpleScopedInjectable,
+                    hints=() if ignore_type_hint else (wrapped,),
+                    wrapper=wrapped,
+                )
 
             self.injectable(
-                wrapper,
-                cls=partial(injectable_class, scope_name=scope_name),
+                ctx.wrapper,
+                cls=ctx.cls.bind_scope_name(scope_name),
                 ignore_type_hint=True,
                 inject=inject,
-                on=(hint, on),
+                on=(*ctx.hints, on),
                 mode=mode,
             )
             return wrapped
@@ -348,6 +349,14 @@ class Module(EventListener, InjectionProvider):  # type: ignore[misc]
 
         return decorator(wrapped) if wrapped else decorator
 
+    def create_metadata[**P, T](
+        self,
+        wrapped: Callable[P, T],
+        /,
+        threadsafe: bool | None = None,
+    ) -> InjectMetadata[P, T]:
+        return InjectMetadata(wrapped, threadsafe).listen(self)
+
     if TYPE_CHECKING:  # pragma: no cover
 
         @overload
@@ -372,12 +381,7 @@ class Module(EventListener, InjectionProvider):  # type: ignore[misc]
         /,
         threadsafe: bool | None = None,
     ) -> InjectedFunction[P, T]:
-        metadata = InjectMetadata(wrapped, threadsafe)
-
-        @metadata.task
-        def listen() -> None:
-            metadata.update(self)
-            self.add_listener(metadata)
+        metadata = self.create_metadata(wrapped, threadsafe)
 
         if iscoroutinefunction(wrapped):
             return AsyncInjectedFunction(metadata)  # type: ignore[arg-type, return-value]
@@ -390,11 +394,7 @@ class Module(EventListener, InjectionProvider):  # type: ignore[misc]
         /,
         threadsafe: bool | None = None,
     ) -> Callable[..., Awaitable[T]]:
-        factory: InjectedFunction[..., T] = self.make_injected_function(
-            wrapped,
-            threadsafe,
-        )
-        return factory.__injection_metadata__.acall
+        return self.create_metadata(wrapped, threadsafe).acall
 
     async def afind_instance[T](
         self,
@@ -507,12 +507,14 @@ class Module(EventListener, InjectionProvider):  # type: ignore[misc]
         *,
         threadsafe: bool | None = None,
     ) -> Awaitable[T | Default]:
-        function = self.make_injected_function(
-            lambda instance=default: instance,
-            threadsafe=threadsafe,
+        return SimpleAwaitable(
+            self.create_metadata(
+                lambda instance=default: instance,
+                threadsafe=threadsafe,
+            )
+            .set_owner(cls)  # type: ignore[arg-type]
+            .acall
         )
-        metadata = function.__injection_metadata__.set_owner(cls)
-        return SimpleAwaitable(metadata.acall)
 
     if TYPE_CHECKING:  # pragma: no cover
 
@@ -541,12 +543,14 @@ class Module(EventListener, InjectionProvider):  # type: ignore[misc]
         *,
         threadsafe: bool | None = None,
     ) -> Invertible[T | Default]:
-        function = self.make_injected_function(
-            lambda instance=default: instance,
-            threadsafe=threadsafe,
+        return SimpleInvertible(
+            self.create_metadata(
+                lambda instance=default: instance,
+                threadsafe=threadsafe,
+            )
+            .set_owner(cls)  # type: ignore[arg-type]
+            .call
         )
-        metadata = function.__injection_metadata__.set_owner(cls)
-        return SimpleInvertible(metadata.call)
 
     def update[T](self, updater: Updater[T]) -> Self:
         self.__locator.update(updater)
@@ -709,18 +713,19 @@ class Module(EventListener, InjectionProvider):  # type: ignore[misc]
         return cls.from_name("__default__")
 
     @staticmethod
-    def __build_key_types(input_cls: Any) -> frozenset[Any]:
+    def __build_key_types(input_class: Any) -> frozenset[Any]:
         config = MatchingTypesConfig(ignore_none=True)
         return frozenset(
-            itertools.chain.from_iterable(
-                iter_matching_types(cls, config) for cls in get_return_types(input_cls)
-            )
+            matching_type
+            for cls in iter_flat_types(input_class)
+            for return_type in iter_return_types(cls)
+            for matching_type in iter_matching_types(return_type, config)
         )
 
     @staticmethod
-    def __matching_key_types(input_cls: Any) -> tuple[Any, ...]:
+    def __matching_key_types(input_class: Any) -> tuple[Any, ...]:
         config = MatchingTypesConfig(with_origin=True, with_type_alias_value=True)
-        return matching_types(input_cls, config)
+        return matching_types(input_class, config)
 
 
 def mod(name: str | None = None, /) -> Module:
@@ -730,22 +735,17 @@ def mod(name: str | None = None, /) -> Module:
     return Module.from_name(name)
 
 
-"""
-InjectedFunction
-"""
-
-
 @dataclass(repr=False, frozen=True, slots=True)
 class Dependencies:
     lazy_mapping: Lazy[Mapping[str, Injectable[Any]]]
 
-    def __iter__(self) -> Iterator[tuple[str, Any]]:
-        for name, injectable in self.items():
+    def iter(self, exclude: Container[str]) -> Iterator[tuple[str, Any]]:
+        for name, injectable in self.items(exclude):
             with suppress(SkipInjectable):
                 yield name, injectable.get_instance()
 
-    async def __aiter__(self) -> AsyncIterator[tuple[str, Any]]:
-        for name, injectable in self.items():
+    async def aiter(self, exclude: Container[str]) -> AsyncIterator[tuple[str, Any]]:
+        for name, injectable in self.items(exclude):
             with suppress(SkipInjectable):
                 yield name, await injectable.aget_instance()
 
@@ -753,19 +753,22 @@ class Dependencies:
     def are_resolved(self) -> bool:
         return self.lazy_mapping.is_set
 
-    async def aget_arguments(self) -> dict[str, Any]:
-        return {key: value async for key, value in self}
+    async def aget_arguments(self, *, exclude: Container[str]) -> dict[str, Any]:
+        return {key: value async for key, value in self.aiter(exclude)}
 
-    def get_arguments(self) -> dict[str, Any]:
-        return dict(self)
+    def get_arguments(self, *, exclude: Container[str]) -> dict[str, Any]:
+        return dict(self.iter(exclude))
 
-    def items(self) -> Iterator[tuple[str, Injectable[Any]]]:
-        return iter((~self.lazy_mapping).items())
+    def items(self, exclude: Container[str]) -> Iterator[tuple[str, Injectable[Any]]]:
+        return (
+            (name, injectable)
+            for name, injectable in (~self.lazy_mapping).items()
+            if name not in exclude
+        )
 
     @classmethod
     def from_iterable(cls, iterable: Iterable[tuple[str, Injectable[Any]]]) -> Self:
-        lazy_mapping = Lazy(lambda: dict(iterable))
-        return cls(lazy_mapping)
+        return cls(Lazy(lambda: dict(iterable)))
 
     @classmethod
     def empty(cls) -> Self:
@@ -858,21 +861,19 @@ class InjectMetadata[**P, T](Caller[P, T], EventListener):
     def wrapped(self) -> Callable[P, T]:
         return self.__wrapped
 
-    async def abind(
-        self,
-        args: Iterable[Any] = (),
-        kwargs: Mapping[str, Any] | None = None,
-    ) -> Arguments:
-        additional_arguments = await self.__dependencies.aget_arguments()
-        return self.__bind(args, kwargs, additional_arguments)
+    async def abind(self, args: Iterable[Any], kwargs: Mapping[str, Any]) -> Arguments:
+        arguments = self.__get_arguments(args, kwargs)
+        if dependencies := await self.__dependencies.aget_arguments(exclude=arguments):
+            return self.__merge_arguments(arguments, dependencies)
 
-    def bind(
-        self,
-        args: Iterable[Any] = (),
-        kwargs: Mapping[str, Any] | None = None,
-    ) -> Arguments:
-        additional_arguments = self.__dependencies.get_arguments()
-        return self.__bind(args, kwargs, additional_arguments)
+        return Arguments(args, kwargs)
+
+    def bind(self, args: Iterable[Any], kwargs: Mapping[str, Any]) -> Arguments:
+        arguments = self.__get_arguments(args, kwargs)
+        if dependencies := self.__dependencies.get_arguments(exclude=arguments):
+            return self.__merge_arguments(arguments, dependencies)
+
+        return Arguments(args, kwargs)
 
     async def acall(self, /, *args: P.args, **kwargs: P.kwargs) -> T:
         with self.__lock:
@@ -911,6 +912,14 @@ class InjectMetadata[**P, T](Caller[P, T], EventListener):
 
         return decorator(wrapped) if wrapped else decorator
 
+    def listen(self, module: Module) -> Self:
+        @self.task
+        def start_listening() -> None:
+            self.update(module)
+            module.add_listener(self)
+
+        return self
+
     @singledispatchmethod
     def on_event(self, event: Event, /) -> ContextManager[None] | None:
         return None
@@ -921,20 +930,19 @@ class InjectMetadata[**P, T](Caller[P, T], EventListener):
         yield
         self.update(event.module)
 
-    def __bind(
+    def __get_arguments(
         self,
         args: Iterable[Any],
-        kwargs: Mapping[str, Any] | None,
-        additional_arguments: dict[str, Any] | None,
+        kwargs: Mapping[str, Any],
+    ) -> dict[str, Any]:
+        return self.signature.bind_partial(*args, **kwargs).arguments
+
+    def __merge_arguments(
+        self,
+        arguments: dict[str, Any],
+        additional_arguments: dict[str, Any],
     ) -> Arguments:
-        if kwargs is None:
-            kwargs = {}
-
-        if not additional_arguments:
-            return Arguments(args, kwargs)
-
-        bound = self.signature.bind_partial(*args, **kwargs)
-        bound.arguments = bound.arguments | additional_arguments | bound.arguments
+        bound = BoundArguments(self.signature, additional_arguments | arguments)  # type: ignore[arg-type]
         return Arguments(bound.args, bound.kwargs)
 
     def __run_tasks(self) -> None:
@@ -943,7 +951,7 @@ class InjectMetadata[**P, T](Caller[P, T], EventListener):
             task()
 
 
-class InjectedFunction[**P, T](HiddenCaller[P, T], ABC):
+class InjectedFunction[**P, T](ABC):
     __slots__ = ("__dict__", "__injection_metadata__")
 
     __injection_metadata__: InjectMetadata[P, T]
@@ -958,10 +966,6 @@ class InjectedFunction[**P, T](HiddenCaller[P, T], ABC):
     def __str__(self) -> str:  # pragma: no cover
         return str(self.__injection_metadata__.wrapped)
 
-    @property
-    def __injection_hidden_caller__(self) -> Caller[P, T]:
-        return self.__injection_metadata__
-
     def __get__(
         self,
         instance: object | None = None,
@@ -975,6 +979,10 @@ class InjectedFunction[**P, T](HiddenCaller[P, T], ABC):
     def __set_name__(self, owner: type, name: str) -> None:
         self.__injection_metadata__.set_owner(owner)
 
+    @abstractmethod
+    def __call__(self, /, *args: P.args, **kwargs: P.kwargs) -> T:
+        raise NotImplementedError
+
 
 class AsyncInjectedFunction[**P, T](InjectedFunction[P, Awaitable[T]]):
     __slots__ = ()
